@@ -2,8 +2,9 @@ from unittest.mock import patch, mock_open, MagicMock
 import sys
 import pytest
 import json
-from io import StringIO
+from io import BytesIO, StringIO
 import io
+from uuid import uuid4
 
 from hop import __version__
 from conftest import temp_environ, temp_config
@@ -82,10 +83,10 @@ def test_cli_publish(script_runner, message_format, message_parameters_dict):
             assert ret.success
         else:  # only the blob format is supported, others should trigger an error
             assert not ret.success
-            assert "piping/redirection only allowed for BLOB formats" in ret.stderr
+            assert "piping/redirection only allowed for BLOB and JSON formats" in ret.stderr
 
 
-def test_cli_publish_blob_types(mock_broker, mock_producer, mock_consumer):
+def test_cli_publish_blob_msgs(mock_broker, mock_producer, mock_consumer):
     from hop import publish, io, models
     import json
     args = MagicMock()
@@ -94,6 +95,48 @@ def test_cli_publish_blob_types(mock_broker, mock_producer, mock_consumer):
     args.test = False
     start_at = io.StartPosition.EARLIEST
     read_url = "kafka://group@hostname:port/topic"
+    fixed_uuid = uuid4()
+
+    mock_adc_producer = mock_producer(mock_broker, "topic")
+    mock_adc_consumer = mock_consumer(mock_broker, "topic", "group")
+    msgs = [b"a string", b"\x10\x00\x20\x0B"]
+    for msg in msgs:
+        with patch("sys.stdin", BytesIO(msg)) as mock_stdin, \
+                patch("hop.io.producer.Producer", return_value=mock_adc_producer), \
+                patch("hop.io.consumer.Consumer", return_value=mock_adc_consumer), \
+                patch("hop.io.uuid4", MagicMock(return_value=fixed_uuid)):
+            publish._main(args)
+
+            # each published message should be on the broker
+            encoded = models.Blob(msg).serialize()
+            expected_msg = {
+                "message": encoded["content"],
+                "headers": [("_id", fixed_uuid.bytes),
+                            ("_format", encoded["format"].encode("utf-8"))],
+            }
+            assert mock_broker.has_message("topic", **expected_msg)
+
+            # reading from the broker should yield messages which match the originals
+            with io.Stream(start_at=None, auth=False).open(read_url, "r") as s:
+                extracted_msgs = []
+                for extracted_msg in s:
+                    extracted_msgs.append(extracted_msg)
+                # there should be one new message
+                assert len(extracted_msgs) == 1
+                # and it should be the one we published
+                assert msg in [msg.content for msg in extracted_msgs]
+
+
+def test_cli_publish_json_blob_msgs(mock_broker, mock_producer, mock_consumer):
+    from hop import publish, io, models
+    import json
+    args = MagicMock()
+    args.url = "kafka://hostname:port/topic"
+    args.format = io.Deserializer.JSON.name
+    args.test = False
+    start_at = io.StartPosition.EARLIEST
+    read_url = "kafka://group@hostname:port/topic"
+    fixed_uuid = uuid4()
 
     mock_adc_producer = mock_producer(mock_broker, "topic")
     mock_adc_consumer = mock_consumer(mock_broker, "topic", "group")
@@ -102,14 +145,16 @@ def test_cli_publish_blob_types(mock_broker, mock_producer, mock_consumer):
     for msg in msgs:
         with patch("sys.stdin", StringIO(json.dumps(msg))) as mock_stdin, \
                 patch("hop.io.producer.Producer", return_value=mock_adc_producer), \
-                patch("hop.io.consumer.Consumer", return_value=mock_adc_consumer):
+                patch("hop.io.consumer.Consumer", return_value=mock_adc_consumer), \
+                patch("hop.io.uuid4", MagicMock(return_value=fixed_uuid)):
             publish._main(args)
 
             # each published message should be on the broker
-            encoded = models.Blob(msg).serialize()
+            encoded = models.JSONBlob(msg).serialize()
             expected_msg = {
                 "message": encoded["content"],
-                "headers": [("_format", encoded["format"].encode("utf-8"))],
+                "headers": [("_id", fixed_uuid.bytes),
+                            ("_format", encoded["format"].encode("utf-8"))],
             }
             assert mock_broker.has_message("topic", **expected_msg)
 
@@ -130,7 +175,7 @@ def test_cli_publish_bad_blob(mock_broker, mock_producer):
 
     args = MagicMock()
     args.url = "kafka://hostname:port/topic"
-    args.format = io.Deserializer.BLOB.name
+    args.format = io.Deserializer.JSON.name
     args.test = False
 
     mock_adc_producer = mock_producer(mock_broker, "topic")
@@ -274,7 +319,7 @@ def test_cli_list_topics(script_runner, auth_config, tmpdir):
         assert "No accessible topics" in ret.stdout
 
         mock_consumer.assert_called()
-        mock_consumer.return_value.list_topics.assert_called_with()
+        mock_consumer.return_value.list_topics.assert_called_with(timeout=-1.)
 
     expected_topics = ["foo", "bar"]
     unexpected_topics = ["baz"]
@@ -297,7 +342,7 @@ def test_cli_list_topics(script_runner, auth_config, tmpdir):
             assert topic not in ret.stdout
 
         mock_consumer.assert_called()
-        mock_consumer.return_value.list_topics.assert_called_with()
+        mock_consumer.return_value.list_topics.assert_called_with(timeout=-1.)
 
     query_topics = ["foo", "bar", "baz"]
     # listing of specific topics, none of which exist
@@ -311,7 +356,7 @@ def test_cli_list_topics(script_runner, auth_config, tmpdir):
 
         mock_consumer.assert_called()
         for topic in query_topics:
-            mock_consumer.return_value.list_topics.assert_any_call(topic=topic)
+            mock_consumer.return_value.list_topics.assert_any_call(topic=topic, timeout=-1.)
 
     # listing of specific topics, some of which exist and some of which do not
     with patch("confluent_kafka.Consumer", make_consumer_mock(topic_results)) as mock_consumer:
@@ -328,7 +373,7 @@ def test_cli_list_topics(script_runner, auth_config, tmpdir):
 
         mock_consumer.assert_called()
         for topic in query_topics:
-            mock_consumer.return_value.list_topics.assert_any_call(topic=topic)
+            mock_consumer.return_value.list_topics.assert_any_call(topic=topic, timeout=-1.)
 
     # general listing with authentication
     with temp_config(tmpdir, auth_config) as config_dir, temp_environ(XDG_CONFIG_HOME=config_dir), \
@@ -344,7 +389,7 @@ def test_cli_list_topics(script_runner, auth_config, tmpdir):
             assert topic not in ret.stdout
 
         mock_consumer.assert_called()
-        mock_consumer.return_value.list_topics.assert_called_with()
+        mock_consumer.return_value.list_topics.assert_called_with(timeout=-1.)
 
     # attempting to use multiple brokers should provoke an error
     ret = script_runner.run("hop", "list-topics", "kafka://example.com,example.net")
