@@ -3,13 +3,16 @@ from dataclasses import asdict, fields
 import json
 import logging
 from pathlib import Path
+import time
 from unittest.mock import patch, MagicMock
+from uuid import uuid4
 
 import pytest
 
 from hop.auth import Auth
 from hop import io
 from hop.models import AvroBlob, Blob, GCNCircular, JSONBlob, VOEvent, format_name
+import confluent_kafka
 
 from conftest import temp_environ, temp_config, message_parameters_dict_data
 
@@ -191,18 +194,24 @@ def test_stream_write(circular_msg, circular_text, mock_broker, mock_producer):
     mock_adc_producer = mock_producer(mock_broker, topic)
     expected_msg = make_message_standard(circular_msg)
 
+    fixed_uuid = uuid4()
+
+    auth = Auth("user", "password")
     headers = {"some header": b"some value", "another header": b"other value"}
     canonical_headers = [("some header", b"some value"),
                          ("another header", b"other value"),
+                         ("_id", fixed_uuid.bytes),
+                         ("_sender", auth.username.encode("utf-8")),
                          ("_format", b"circular")]
     test_headers = canonical_headers.copy()
-    test_headers.insert(2, ('_test', b'true'))
-    none_test_headers = [('_test', b"true"), ("_format", b"circular")]
+    test_headers.insert(4, ('_test', b'true'))
+    none_test_headers = [("_id", fixed_uuid.bytes), ("_sender", auth.username.encode("utf-8")),
+                         ('_test', b"true"), ("_format", b"circular")]
 
-    with patch("hop.io.producer.Producer", autospec=True, return_value=mock_adc_producer):
+    with patch("hop.io.producer.Producer", autospec=True, return_value=mock_adc_producer), \
+            patch("hop.io.uuid4", MagicMock(return_value=fixed_uuid)):
 
         broker_url = f"kafka://localhost:port/{topic}"
-        auth = Auth("user", "password")
         start_at = io.StartPosition.EARLIEST
         until_eos = True
 
@@ -548,7 +557,7 @@ def make_mock_listing_consumer(topics=[]):
         which has a list_topics method which acts as if a predetermined set of
         topics exist.
     """
-    def get_topics(topic=None):
+    def get_topics(topic=None, timeout=None):
         nonlocal topics
         result = {}
         if topic is None:
@@ -674,3 +683,13 @@ def test_list_topics_auth(auth_config, tmpdir):
         assert("sasl.password" in cons_args[0])
         assert(cons_args[0]["sasl.username"] == "user2")
         assert(cons_args[0]["sasl.password"] == "pass2")
+
+
+def test_list_topics_timeout():
+    cred = Auth("user", "pass")
+    for timeout in [0.5, 1.0, 1.5]:
+        start = time.time()
+        with pytest.raises(confluent_kafka.KafkaException) as err:
+            io.list_topics("kafka://not-a-valid-broker.scimma.org", auth=cred, timeout=timeout)
+        stop = time.time()
+        assert(abs((stop - start) - timeout) < 0.1)
